#### Code for SDM and niche change analysis of Forficula auricularia
#### Matt Hill 2018-07-27
#### matthill@protonmail.com
#### Version for Submission to Pest Management Science

library (rgeos)
library (dismo)
library (maptools)
library (ggplot2)
library (rgbif)
library (rJava)
library (reshape2)
library (rasterVis)
library (viridis)
library (corrplot)

#######################################################################
# Begin data prep

##First, lookup the species on GBIF using the rgbif package
head(name_lookup(query = 'Forficula auricularia', rank="species", return = 'data'))
hold.dat <- occ_search(scientificName = "Forficula auricularia", limit = 20)

#7862 records on the 27/3/2017

key <- name_backbone(name='Forficula auricularia')$speciesKey
# download all the records and plot (based on a ggplot function)
dat <- occ_search(taxonKey=key, return='data', limit=hold.dat$meta$count)
gbifmap(dat)

dist <- cbind (dat[,c("species",  "decimalLatitude", "decimalLongitude")])

### ALA data
library (ALA4R)
y <- occurrences(taxon="forficula auricularia",fields=c("latitude","longitude", "basis_of_record"),download_reason_id=10)

ala_dist <- data.frame(cbind("species" = "Forficula auricularia", y$data[, c("latitude", "longitude")]))
ala_dist <- ala_dist[complete.cases(ala_dist),]
colnames (ala_dist) <- c("species",  "decimalLatitude", "decimalLongitude") ## check the two points that appear to be noether hemispere...

#### add both together
dist <- rbind (dist, ala_dist)

#### Note: we had additional data sources APPD data comes from a closed database and I had to gain permission to use this data - not for redistribution.
#### We also obained data from Quarrell et al. 2018. See main text for details.
#### Those data were added to the file

write.csv (dist, file="complete_dist.csv", row.names = FALSE)
### End of data prep


## Read in from here
dist <- read.csv("complete_dist.csv", header=T)

## These data are provided with this file - colected as part of this study.
## Note the data available on FigShare have already been rescaled to 
grains_dist <- read.csv("grains_dist.csv", header=T)

## Predictor varibles
## stacked Worldclim 2.0 data in 10' resolution
bioclimall <- stack("./worldclim/bioclimall_clipped.grd")

## human influence index (Sanderson et al. 2002)
hii <- raster ("hii_10min.tif")
hii <- resample (hii, bioclimall[[1]])
names(hii) <- "hii"

## Aridity layer is from CGIAR
arid <- raster("./arid.grd")
names(arid) <- "arid"
bioclimall <- stack(bioclimall, arid, hii)

## This can be any world outline polygon
worldshp <- readShapePoly ("./worldadmin/WorldAdmin_Clip1.shp") 

## Figure S1.1 - Map of Australia with our occurrence points

ggplot()+
  geom_polygon(data=wrld_simpl, aes(x=long, y=lat, group=group), alpha=0.5)+
  geom_path (data=wrld_simpl, aes(x=long, y=lat, group=group))+
  #geom_point(data=austdist, aes(decimalLongitude, decimalLatitude), size=3, alpha=.75, shape = 21, colour = "black", fill = "light blue")+
  geom_point(data=grains_dist, aes(decimalLongitude, decimalLatitude), size=3, alpha=.75, shape = 21, colour = "black", fill = "light blue")+
  coord_equal()+
  ## Set limits to the plotting area (Australia)
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10))+
  geom_path (data=worldshp, aes(x=long, y=lat, group=group))+
  ## Some manual adding of State and Territory labels
  annotate("text", x = 122, y = -25.5, label = "Western Australia")+
  annotate("text", x = 135, y = -30, label = "South Australia")+
  annotate("text", x = 148.5, y = -38.5, label = "Victoria")+
  annotate("text", x = 143, y = -42, label = "Tasmania")+
  annotate("text", x = 144.5, y = -32, label = "New South Wales")+
  annotate("text", x = 144, y = -23, label = "Queensland")+
  annotate("text", x = 133.5, y = -20, label = "Northern Territory")+
  xlab("Longitude")+
  ylab("Latitude")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12),
        axis.title.y = element_text(size=14),
        axis.title.x = element_text(size=14),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))


##### Seeting up prediction data############
## Need the extent of Australia for defining the ranges
# Predict the model to the climate values for Australia (data.frame)
e <- extent (112, 155, -45, -10)
Australia <- crop (bioclimall, e)
#convert to a stack
Australia <- stack(unstack(Australia))

### Data cleaning
## There are number of points that seem a bit odd..
#A bounding box of x = -130, 40 and y = 20, 75 should clear them up...

austdist <- dist[dist$decimalLongitude > 112 & dist$decimalLongitude < 155, ]
austdist <- austdist[austdist$decimalLatitude > -45 & austdist$decimalLatitude < -10, ]

globdist <- dist[dist$decimalLongitude > -130 & dist$decimalLongitude < 40, ]
globdist <- globdist[globdist$decimalLatitude > 20 & globdist$decimalLatitude < 75, ]

NZdist <- dist[dist$decimalLongitude > 164 & dist$decimalLongitude < 180, ]
NZdist <- NZdist[NZdist$decimalLatitude > -55 & NZdist$decimalLatitude < -33, ]

dist <- rbind (globdist, NZdist, austdist)

#############################################
###LOAD backgrounds

inv_ras <- raster("earwigs_Australia.grd")
inv_ras <- crop (inv_ras,e)

nat_ras <- raster("earwigs_global.grd")
#nat_ras <- crop(nat_ras, c(-180, 75, 0, 100))


if (file.exists("earwigs_global_alldata.grd")){
  alldata <- raster ("earwigs_global_alldata.grd")
} else{
  temp <- resample (inv_ras, bioclimall[[1]])
  temp2 <- resample (nat_ras, bioclimall[[1]])
  alldata <-sum(stack(list(temp, temp2)), na.rm=T)
  alldata <- reclassify(alldata, c(-Inf, 0, NA))
  alldata <- trim (alldata)
  writeRaster(alldata, filename="earwigs_global_alldata.grd", overwrite=TRUE)
}

##### Additional Data formatting 

##  create a reference raster from this, rescale the points to 1 per grid cell 
##  and create a new data.frame with this new distribution dataset.
refrast <- biovar[[1]]
sppR <- rasterize (dist[,3:2], refrast)
sppG <- rasterize (grains_dist[,3:2], refrast)

## for intial models we want to use ALL DATA
sppP <- data.frame (rbind(rasterToPoints (sppR), rasterToPoints(sppG)))

### To show now we only have 1 unique point per grid cell:
length (rbind(dist[,1], grains_dist[,1]))
length (sppP[,1])

## load the worldmap from the maptools package
data("wrld_simpl")

# Plot the shapefile and points to see how it looks...
plot (wrld_simpl)
points (sppP[,1:2], pch=20, col="red")

######## Create the species data frame
sppP$pa <- 1
clim.df <- as.data.frame(extract (biovar, sppP[,1:2]))
sppDF <- cbind (sppP, clim.df)
sppDF$layer <- NULL

######## Perform a PCA on the presence data
code <- extract (Australia[[1]], dist[,3:2])
code[is.na(code)] <- "Global"
code[code!="Global"] <- "Australia"

grains_code <- rep("grains", nrow(grains_dist))
code <- c(code, grains_code)

pca_dat <- as.data.frame(extract (bioclimall, rbind(dist[,3:2], grains_dist[,3:2])))
pca_dat <- cbind(pca_dat, code)
pca_dat <- pca_dat[complete.cases(pca_dat),]

pca_out <- princomp(pca_dat[,1:20]) ## just climatic data

pca.dat <- as.data.frame(pca_out$scores)

pca_dat <- as.data.frame(pca_dat)


pca1 <- ggplot (pca.dat, aes(Comp.1, Comp.2))+
  geom_point(alpha=.2, size=.5)+
  scale_fill_manual("legend", values = viridis(3))+
  ggConvexHull::geom_convexhull(alpha=0.5, aes(fill=pca_dat$code))+
  theme_minimal()+
  xlab("Component 1")+
  ylab("Component 2")+
  ggtitle("A")+
  guides(fill=guide_legend(title="Dataset"))

pca2 <- ggplot (pca.dat[pca_dat$code == "Australia" | pca_dat$code == "grains",], aes(Comp.1, Comp.2, fill=pca_dat$code[pca_dat$code == "Australia" | pca_dat$code == "grains"]))+

  geom_point(alpha=0.6, size=2)+
  scale_fill_manual("legend", values = viridis(2))+
  ggConvexHull::geom_convexhull(alpha=0.3)+
  theme_minimal()+
  ggtitle ("PCA of Australian dataset")

components <-summary (pca_out)
components

# with aridity, this accounts for 99.8% variance on comp1 
#withou aridity ## Comp1 = 84.4% variance, Comp2 = 13% deviance

components$loadings
#Comp 1 = #bio04, bio12, bio13, bio16, bio17, bio18, bio19
#Comp 2 = #bio04, bio12, bio18, bio19


###############3 FIgure 1
##### look at null dist and associations with HII 
background.ras <- alldata
null_spp <- randomPoints(background.ras, nrow(sppDF))
null_sppDF <- as.data.frame(extract(biovar, null_spp))
null_sppDF <- cbind (null_spp, null_sppDF)
null_sppDF$species <- "Null"

sppDF2 <- sppDF
sppDF2$pa <-NULL
sppDF2$species <- "Actual"

dens_data <- rbind (sppDF2, null_sppDF)

dens_data <- melt(dens_data, id.vars = c("x", "y", "species"))

ggplot (dens_data)+
  geom_density(aes(x=value, fill=species), alpha=0.5)+
  facet_wrap(~variable, scales="free")

hiidens <- ggplot (dens_data[dens_data$variable=="hii",])+
  geom_density(aes(x=value, fill=species), alpha=0.5)+
  scale_fill_manual("legend", values = viridis(2))+
  theme_minimal()+
  xlab("HII score")+
  ylab("Density")+
  ggtitle("B")+
  guides(fill=guide_legend(title="Distribution"))
  
## Figure 1 plot

pdf("Figure1.pdf", width = 10, height=5)
gridExtra::grid.arrange(pca1, hiidens, ncol=2, widths=c(3, 2))
dev.off()

keep_vars <-  c("arid",
                "bio04",
                "bio12",
                "bio13",
                "bio16",
                "bio17",
                "bio18",
                "bio19",
                "hii")

### examine correlations (across training background rather than presence points)
background <- as.data.frame(randomPoints(alldata, 40000))

background.df <- as.data.frame(extract (bioclimall[[keep_vars]], background[,1:2]))
background.df <- background.df[complete.cases(background.df),]

pca_cor <- cor(background.df, method="kendall") ## kendall method takes a while compared to Pearson and Spearman...
corrplot (pca_cor, method="number")

#bio12 & bio13 correlated at 0.81
#bio12 & bio16 correlated at 0.84
#bio12 & bio17 correlated at 0.69
#bio12 & bio18 correlated at 0.69
#bio12 & bio19 correlated at 0.65

#bio13 & bio18 correlated at 0.78
#bio13 $ bio19 correlated at 0.5

#bio16 & bio18 correlated at 0.79
#bio16 & bio19 correlated at 0.52

#bio17 & bio19 correlated at 0.77

### Keeping these correlations in mind, running the models first and revising after seems best way to go...

biovar <- subset(bioclimall, keep_vars)
Australia <- subset(Australia, keep_vars)

###################################################################################################
## Species Distribution Modelling

background$pa <- 0
background.df <- as.data.frame(extract (biovar, background[,1:2]))
bgDF <- cbind (background, background.df)

background.ras <- alldata
###################################################################################################
library (dismo)
library (rmaxent) 

jar <- paste("./maxent/maxent.jar")

file.exists(jar) ## must be true!

wd <- getwd()

## use only the presence points relevant to the model background
in_or_out <- extract(background.ras, sppDF[,1:2])
pres.env <- sppDF[,4:ncol(sppDF)][which (in_or_out==1),]

bg.env <- background.df
pbg.env <- rbind(pres.env, bg.env)
pbg.which <- c(rep(1, nrow(pres.env)), rep(0, nrow(bg.env)))

#default settings

max1SWD <- maxent(x=pbg.env, p=pbg.which, path=paste(wd,'/max1SWD',sep=""),
                  args=c("beta_threshold=-1", 
                         "-P",
                         "outputformat=logistic"))

# default with cross-validation
max1SWD.cv <- maxent(x=pbg.env, p=pbg.which, path=paste(wd,'/max1SWD',sep=""),
                  args=c("beta_threshold=-1", 
                         "-P",
                         "replicates=10",
                         "replicatetype=crossvalidate",
                         "outputformat=logistic"))

## These are the point process settings to be used for the subsequent models
arg.list <- c("beta_threshold=-1", 
              "-P", 
              "noautofeature", 
              "nothreshold", 
              "noproduct",
              "maximumbackground=40000",
              "noaddsamplestobackground",  
              "noremoveduplicates",
              "outputformat=logistic")

## point-process settings 
max2SWD <- maxent(x=pbg.env, p=pbg.which, path=paste(wd,'/max2SWD',sep=""),
                  args=arg.list)

max2SWD.cv <- maxent(x=pbg.env, p=pbg.which, path=paste(wd,'/max2SWD',sep=""),
                  args=c(arg.list, 
                         "replicates=10",
                         "replicatetype=crossvalidate"))

## point process - climate only 

max3SWD <- maxent(x=pbg.env[,1:8], p=pbg.which, path=paste(wd,'/max3SWD',sep=""),
                  args=arg.list)

max3SWD.cv <- maxent(x=pbg.env[,1:8], p=pbg.which, path=paste(wd,'/max3SWD',sep=""),
                     args=c(arg.list, 
                            "replicates=10",
                            "replicatetype=crossvalidate"))

## Look at evaluation on the training dataset
max1SWD.cv@results["Training.AUC",11]
max2SWD.cv@results["Training.AUC",11]
max3SWD.cv@results["Training.AUC",11]
# Point process model does the better job

me1 <- project (max1SWD, biovar)
me2 <- project (max2SWD, biovar)
me3 <- project (max3SWD, biovar)

occ <- sppDF[,1:2]
colnames (occ) <- c("lon", "lat")

ic(stack(me1$prediction_raw, me2$prediction_raw, me3$prediction_raw), 
   occ, list(max1SWD, max2SWD, max3SWD))

################## Limiting factor analysis
lim1 <- limiting(Australia, max2SWD)
levelplot(lim1, col.regions=rainbow) +
  layer(sp.points(SpatialPoints(occ), pch=20, col=1))

## Examine the limiting factor analysis, and then the percent contribution and permutation importance as determined by maxent
## also examine the correlations examined before

#bio12, bio16, could be dropped based on permutation importance
#bio13 is correlated with 18, but is slightly more important - so 18 is dropped
#bio17 is correlated with 19, but lower importance, 17 is dropped
corrplot (pca_cor, method="number")
#update keep_vars # these are best performing, uncorrelated variables 
final_vars <-  c("arid",
                 "bio04",
                 "bio19",
                 "bio13",
                 "hii")

## point-process settings 
max4SWD <- maxent(x=pbg.env[,c(final_vars)], p=pbg.which, path=paste(wd,'/max4SWD',sep=""),
                  args=arg.list)

max4SWD.cv <- maxent(x=pbg.env[,c(final_vars)], p=pbg.which, path=paste(wd,'/max4SWD',sep=""),
                  args=c(arg.list,
                         "replicates=10",
                         "replicatetype=crossvalidate"))

max4SWD.cv@results["Training.AUC",11]

me4 <- project (max4SWD, biovar)

ic(stack(me2$prediction_raw, me4$prediction_raw), 
   occ, list(max2SWD, max4SWD))

### the new model gives a lower AICc/

lim2 <- limiting(Australia, max4SWD)
levelplot(lim2, col.regions=rainbow) +
  layer(sp.points(SpatialPoints(occ), pch=20, col=1))


####### Plot up the two map quickly....

max2.pred <- predict(max2SWD, Australia, args="outputformat=logistic",
                     filename=paste(wd, '/max2SWD/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                     progress="text")

max3.pred <- predict(max3SWD, Australia, args="outputformat=logistic",
                     filename=paste(wd, '/max3SWD/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                     progress="text")


max4.pred <- predict(max4SWD, Australia, args="outputformat=logistic",
                     filename=paste(wd, '/max4SWD/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                     progress="text")

gplot(max2.pred) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1)+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10)) +
  ggtitle (paste0("Forficula auricularia Maxent (point process) SDM AUC = ", max2SWD.cv@results["Training.AUC",11]))

gplot(max3.pred) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1)+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10)) +
  ggtitle (paste0("Forficula auricularia Maxent (point process) SDM AUC = ", max3SWD.cv@results["Training.AUC",11]))

gplot(max4.pred) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1)+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10)) +
  ggtitle (paste0("Forficula auricularia Maxent (point process) SDM AUC = ", max4SWD.cv@results["Training.AUC",11]))



## point-process settings 
max5SWD <- maxent(x=pbg.env[,c(final_vars[1:4])], p=pbg.which, path=paste(wd,'/max5SWD',sep=""),
                  args=arg.list)

max5SWD.cv <- maxent(x=pbg.env[,c(final_vars[1:4])], p=pbg.which, path=paste(wd,'/max5SWD',sep=""),
                  args=c(arg.list,
                  "replicates=10",
                  "replicatetype=crossvalidate"))


max5.pred <- predict(max5SWD, Australia, args="outputformat=logistic",
                     filename=paste(wd, '/max5SWD/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                     progress="text")

me5 <- project (max5SWD, biovar)

gplot(max5.pred) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1)+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10)) +
  ggtitle (paste0("Forficula auricularia Maxent (point process) SDM AUC = ", max5SWD.cv@results["Training.AUC",11]))


results <- as.data.frame(ic(stack(me1$prediction_raw, me2$prediction_raw, me3$prediction_raw, me4$prediction_raw, me5$prediction_raw), 
   occ, list(max1SWD, max2SWD, max3SWD, max4SWD, max5SWD)))
results$AUC <- NA
results$AUC.sd <-NA 
results$vars <- NA
results$AUC[1] <- max1SWD.cv@results["Training.AUC",11]; results$AUC.sd[1] <- max1SWD.cv@results["AUC.Standard.Deviation",11] ; results$vars[1] <- paste(keep_vars, collapse=" ")
results$AUC[2] <- max2SWD.cv@results["Training.AUC",11]; results$AUC.sd[2] <- max2SWD.cv@results["AUC.Standard.Deviation",11] ; results$vars[2] <- paste(keep_vars, collapse=" ")
results$AUC[3] <- max3SWD.cv@results["Training.AUC",11]; results$AUC.sd[3] <- max3SWD.cv@results["AUC.Standard.Deviation",11] ; results$vars[3] <- paste(keep_vars[1:8], collapse=" ")
results$AUC[4] <- max4SWD.cv@results["Training.AUC",11]; results$AUC.sd[4] <- max4SWD.cv@results["AUC.Standard.Deviation",11] ; results$vars[4] <- paste(final_vars, collapse=" ")
results$AUC[5] <- max5SWD.cv@results["Training.AUC",11]; results$AUC.sd[5] <- max5SWD.cv@results["AUC.Standard.Deviation",11] ; results$vars[5] <- paste(final_vars[1:4], collapse=" ")

#### Table for paper
write.csv(results, file="./model_summ.csv")

#########################################################################################################################################
## FIGURE PLOT

# hay: long=144.85 lat=-34.5 (Referred to in text)

gplot(max4.pred) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1, "Suitability", na.value=NA)+
  geom_path (data=wrld_simpl, aes(x=long, y=lat, group=group))+
  geom_point(data=austdist, aes(decimalLongitude, decimalLatitude), size=3, alpha=.75, shape = 21, colour = "black", fill = "light blue")+
  geom_point(data=grains_dist, aes(decimalLongitude, decimalLatitude), size=3, alpha=.75, shape = 21, colour = "black", fill = "light blue")+
  coord_equal()+
  ## Set limits to the plotting area (Australia)
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10))+
  geom_path (data=worldshp, aes(x=long, y=lat, group=group))+
  ## Some manual adding of State and Territory labels
  annotate("text", x = 122, y = -25.5, label = "Western Australia")+
  annotate("text", x = 135, y = -30, label = "South Australia")+
  annotate("text", x = 148.5, y = -38.5, label = "Victoria")+
  annotate("text", x = 143, y = -42, label = "Tasmania")+
  annotate("text", x = 144.5, y = -32, label = "New South Wales")+
  annotate("text", x = 144, y = -23, label = "Queensland")+
  annotate("text", x = 133.5, y = -20, label = "Northern Territory")+
  annotate("text", x = 144.85, y = -33.8, label = "Hay", size=3.5)+
  xlab("Longitude")+
  ylab("Latitude")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12),
        axis.title.y = element_text(size=14),
        axis.title.x = element_text(size=14),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))

  

ggsave("Figure2.pdf")


## Response curve plots
varlist <- c("arid", "bio04", "bio13", "bio19", "hii")


out <- data.frame()

for (spp in 0:9){
  for (var in varlist){
    temp <- read.csv(paste0("./max4SWD/plots/species_",spp,"_", var,".dat"))
    temp <- cbind ("species" = paste0("Species",spp), temp)
    out <- rbind (out, temp)
  }
}

ggplot (out, aes(x, y))+
  geom_line(alpha=0.5, color=viridis(2)[2])+
  stat_summary(fun.y=mean, colour=viridis(2)[1], geom="line",group=1)+
  facet_wrap(~variable, scales="free")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12, face="bold"),
        strip.text = element_text(size=14),
        axis.title.y = element_text(size=14),
        axis.title.x = element_blank(),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))+
  ylab("Logisitic output")


#==============================================================================================================================
## Australia modelling - quick test in the appendix

soil <- raster ("./data/soil/w001001.adf")

soil <- resample (soil, Australia[[1]]); names(soil) <- "soil"

Auspred <- stack(Australia[[final_vars]], soil)

max.auspred <- maxent (x=Auspred, p=cbind(austdist$decimalLongitude, austdist$decimalLatitude),
                       args=arg.list)

auspred.pred <- predict(max.auspred, Auspred, args="outputformat=logistic",
                     filename=paste(wd, '/auspred/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                     progress="text")

###############################################################################################################################
## Reciprocal distribution modelling


# same as before, except now we only use the background thats everywhere but Australia
# selection of variables and model conditions is all done prior
# this is see how a subset of the distribution data can reciprocally model across
# we expect that Australia will underpredict the global distribution

background.ras <- nat_ras
background <- as.data.frame(randomPoints(background.ras, 40000))
background$pa <- 0
background.df <- as.data.frame(extract (biovar[[final_vars]], background[,1:2]))
bgDF <- cbind (background, background.df)

in_or_out <- extract(background.ras, sppDF[,1:2])
pres.env <- sppDF[,c(final_vars)][which (in_or_out==1),]

bg.env <- background.df
pbg.env <- rbind(pres.env, bg.env)
pbg.which <- c(rep(1, nrow(pres.env)), rep(0, nrow(bg.env)))

Glob2Aus <- maxent(x=pbg.env, p=pbg.which, path=paste(wd,'/Glob2Aus',sep=""),
                  args=c("beta_threshold=-1", 
                         "-P", #response curves
                         "noautofeature", 
                         "nothreshold", 
                         "noproduct",
                         "maximumbackground=40000",
                         "noaddsamplestobackground",  
                         "noremoveduplicates",
                         "replicates=10",
                         "replicatetype=crossvalidate"))

Glob2Aus.pred <- predict(Glob2Aus, biovar, args="outputformat=logistic",
                     filename=paste(wd, '/Glob2Aus/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                     progress="text")

### Aus to Global

background.ras <- inv_ras
background <- as.data.frame(randomPoints(background.ras, 40000))
background$pa <- 0
background.df <- as.data.frame(extract (biovar[[final_vars]], background[,1:2]))
bgDF <- cbind (background, background.df)

in_or_out <- extract(background.ras, sppDF[,1:2])
pres.env <- sppDF[,c(final_vars)][which (in_or_out==1),]

bg.env <- background.df
pbg.env <- rbind(pres.env, bg.env)
pbg.which <- c(rep(1, nrow(pres.env)), rep(0, nrow(bg.env)))

Aus2Glob <- maxent(x=pbg.env, p=pbg.which, path=paste(wd,'/Aus2Glob',sep=""),
                   args=c("beta_threshold=-1", 
                          "-P", #response curves
                          "noautofeature", 
                          "nothreshold", 
                          "noproduct",
                          "maximumbackground=40000",
                          "noaddsamplestobackground",  
                          "noremoveduplicates",
                          "replicates=10",
                          "replicatetype=crossvalidate"))

Aus2Glob.pred <- predict(Aus2Glob, biovar, args="outputformat=logistic",
                         filename=paste(wd, '/Aus2Glob/pred.asc', sep=""), format="ascii", overwrite=TRUE,
                         progress="text")

## AUC and Schoener's D testing.
## raster preparation
inv_ras <- crop (inv_ras, Australia)
inv_ras <- resample(inv_ras, biovar[[1]])
nat_ras <- resample(nat_ras, biovar[[1]])

Glob2Aus.pred1 <- trim(mask(Glob2Aus.pred, inv_ras))
Glob2Aus.pred2 <- trim(mask(Glob2Aus.pred, nat_ras))

Glob2Aus.pred1 <- mean(Glob2Aus.pred1)
Glob2Aus.pred2 <- mean(Glob2Aus.pred2)

Aus2Glob.pred1 <- trim(mask(Aus2Glob.pred, inv_ras))
Aus2Glob.pred2 <- trim(mask(Aus2Glob.pred, nat_ras))
Aus2Glob.pred1 <- mean(Aus2Glob.pred1)
Aus2Glob.pred2 <- mean(Aus2Glob.pred2)


### Global projecting to Australia:
## AUC
p <- rasterize (sppP[,1:2],Glob2Aus.pred1)
p <- mask(Glob2Aus.pred1, p)
a <- mask(Glob2Aus.pred1, p, inverse=TRUE)
p <- rasterToPoints(p)[,3]
a <- rasterToPoints(a)[,3]

evaluate (p, a)

## Schoener's D
nicheOverlap(Glob2Aus.pred1, Aus2Glob.pred1)

## ArcGIS base file
worldshp <- readShapePoly ("./worldadmin/WorldAdmin.shp") 
## Clip just to Australia
Aust <- worldshp[worldshp@data$CNTRY_NAME=="Australia",]

RDM.B <- gplot(Glob2Aus.pred1) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1, na.value = NA,
                     breaks=seq(0, 1, 0.25), limits=c(0,1))+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10)) +
  geom_path (data=worldshp, aes(x=long, y=lat, group=group))+
  ggtitle("B. Global model projected to Australia")+
  xlab("Longitude")+
  ylab("Latitude")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12),
        axis.title.y = element_text(size=14),
        axis.title.x = element_text(size=14),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))+
  theme(legend.title=element_blank())

RDM.D <- gplot(Aus2Glob.pred1) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1, na.value = NA,
                     breaks=seq(0, 1, 0.25), limits=c(0,1))+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(112, 155)) +
  scale_y_continuous(expand = c(0,0), limits= c(-45, -10)) +
  geom_path (data=worldshp, aes(x=long, y=lat, group=group))+
  ggtitle("D. Australia model projected to Australia")+
  xlab("Longitude")+
  ylab("Latitude")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12),
        axis.title.y = element_text(size=14),
        axis.title.x = element_text(size=14),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))+
  theme(legend.title=element_blank())


##Australia projecting globally
## AUC
p <- rasterize (sppP[,1:2],Aus2Glob.pred2)
p <- mask(Aus2Glob.pred2, p)
a <- mask(Aus2Glob.pred2, p, inverse=TRUE)
p <- rasterToPoints(p)[,3]
a <- rasterToPoints(a)[,3]

evaluate (p, a)

## Schoener's D
nicheOverlap(Aus2Glob.pred2, Glob2Aus.pred2)

RDM.C <- gplot(Aus2Glob.pred2) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1, na.value = NA,
                     breaks=seq(0, 1, 0.25), limits=c(0,1))+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(-180, 75)) +
  scale_y_continuous(expand = c(0,0), limits= c(3.833333, 80)) +
  geom_path (data=worldshp, aes(x=long, y=lat, group=group), alpha=0.5, size=0.1)+
  ggtitle("C. Australia model projected to Global model")+
  xlab("Longitude")+
  ylab("Latitude")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12),
        axis.title.y = element_text(size=14),
        axis.title.x = element_text(size=14),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))+
  theme(legend.position="none")

RDM.A <- gplot(Glob2Aus.pred2) + geom_tile(aes(fill = value)) +
  scale_fill_viridis(option="magma", begin=0.0, direction=-1, na.value = NA,
                     breaks=seq(0, 1, 0.25), limits=c(0,1))+
  coord_equal()+
  scale_x_continuous(expand = c(0,0), limits= c(-180, 75)) +
  scale_y_continuous(expand = c(0,0), limits= c(3.833333, 80)) +
  geom_path (data=worldshp, aes(x=long, y=lat, group=group), alpha=0.5, size=0.1)+
  ggtitle("A. Global model projected to Global model")+
  xlab("Longitude")+
  ylab("Latitude")+
  theme_minimal()+
  theme(axis.text.y=element_text(size =12),
        axis.title.y = element_text(size=14),
        axis.title.x = element_text(size=14),
        axis.text.x=element_text(size =12),
        legend.title = element_text(size=14),
        legend.text=element_text(size=12))+ 
  theme(legend.position="none")



cowplot::plot_grid(RDM.A, RDM.B, RDM.C, RDM.D,
          rel_widths = c(1.75, 1))

################################################################################################################################
## Niche shift analysis

## Coding - Hypothesis 1 - Australia vs Global
locs.dat <- (sppP[,1:2])
coordinates (locs.dat) <- ~x+y
wrld_simpl@proj4string <- CRS("+proj=longlat +datum=WGS84")
locs.dat@proj4string <- CRS("+proj=longlat +datum=WGS84")

code <- extract (Australia[[1]], locs.dat)
code[is.na(code)] <- 0
code[code!=0] <- 1

sppP$code <- code

library (ecospat)

occ.sp1 <- sppDF[sppP$code==0,c("x", "y", "pa", final_vars)]
occ.sp2 <- sppDF[sppP$code==1,c("x", "y", "pa", final_vars)]

occ.sp1 <- occ.sp1[complete.cases(occ.sp1),]
occ.sp2 <- occ.sp2[complete.cases(occ.sp2),]

x1 <- randomPoints(nat_ras, 10000)
clim1 <- as.data.frame(cbind(x1,(extract(biovar[[final_vars]], x1))))
clim1 <- clim1[complete.cases(clim1),]

x2 <- randomPoints(inv_ras, 10000)
clim2 <- as.data.frame(cbind(x2,(extract(biovar[[final_vars]], x2))))
clim2 <- clim2[complete.cases(clim2),]

clim1$pa <- 0
clim2$pa <- 0

occ.sp1$code <- NULL
occ.sp2$code <- NULL

inv <- rbind (occ.sp2, clim2)
nat <- rbind (occ.sp1, clim1)


## Hii is the last column, remove to do analysis without (Figure 4b)
## comment out both lines to do the comparison without HII included (Figure 4a)
inv <- inv[,1:ncol(inv)-1]
nat <- nat[,1:ncol(nat)-1]


pca.env <- dudi.pca(nat[,4:ncol(nat)][nat$pa==1,],scannf=F,nf=2)
pca.env2 <- dudi.pca(inv[,4:ncol(inv)][inv$pa==1,],scannf=F,nf=2)
niceOverPlot(pca.env, pca.env2, n1=nrow(pca.env$li) , n2= nrow(pca.env2$li)) 



pca.env <- dudi.pca(rbind(nat,inv)[,4:ncol(nat)],scannf=F,nf=2)
ecospat.plot.contrib(contrib=pca.env$co, eigen=pca.env$eig)

scores.globclim <- pca.env$li

scores.sp.nat <- suprow(pca.env,nat[which(nat[,3]==1),4:ncol(nat)])$li
scores.sp.inv <- suprow(pca.env,inv[which(inv[,3]==1),4:ncol(nat)])$li

scores.clim.nat <-suprow(pca.env,nat[,4:ncol(nat)])$li
scores.clim.inv <-suprow(pca.env,inv[,4:ncol(nat)])$li

grid.clim.nat <- ecospat.grid.clim.dyn (glob=scores.globclim,
  glob1=scores.clim.nat,
  sp=scores.sp.nat, R=100,
  th.sp=0)

grid.clim.inv <- ecospat.grid.clim.dyn(glob=scores.globclim,
  glob1=scores.clim.inv,
  sp=scores.sp.inv, R=100,
  th.sp=0)

D.overlap <- ecospat.niche.overlap(grid.clim.nat, grid.clim.inv, cor=T)$D
D.overlap

niche.dyn <- ecospat.niche.dyn.index(grid.clim.nat, grid.clim.inv, intersection = 0.1)
niche.dyn 


## output file
## change to a or b depending on test (inclusion of HII)

pdf(file = "Fig 4b.pdf", paper = "special", height=8, width=8)
ecospat.plot.niche.dyn(grid.clim.nat, grid.clim.inv, quant=0.25, interest=2, 
                       colz1="#66C2A5", colz2="#FC8D62", colZ1="#1B9E77", colZ2="#D95F02")
                       #colinter="#8DA0CB")
ecospat.shift.centroids(scores.sp.nat, scores.sp.inv, scores.clim.nat, scores.clim.inv)

dev.off()

eq.test <- ecospat.niche.equivalency.test(grid.clim.nat, grid.clim.inv,rep=100, alternative = "greater")
sim.test <- ecospat.niche.similarity.test (grid.clim.nat, grid.clim.inv, rep=100, alternative = "greater",rand.type=2)
ecospat.plot.overlap.test(eq.test, "D", "Equivalency")
ecospat.plot.overlap.test(sim.test, "D", "Similarity")


 for (i in names(nat[,4:ncol(nat)])){
 # gridding the native niche
 grid.clim.t.nat <-ecospat.grid.clim.dyn(glob=as.data.frame(rbind(nat,inv)[,i]),
                                         glob1=as.data.frame(nat[,i]),sp=as.data.frame(nat[which(nat[,3]==1),i]), R=1000, th.sp=0)
 # gridding the invaded niche
 grid.clim.t.inv <-ecospat.grid.clim.dyn(glob=as.data.frame(rbind(nat,inv)[,i]),
                                         glob1=as.data.frame(inv[,i]),sp=as.data.frame(inv[which(inv[,3]==1),i]), R=1000, th.sp=0)

 t.dyn<-ecospat.niche.dyn.index(grid.clim.t.nat, grid.clim.t.inv,intersection=0.1)
 ecospat.plot.niche.dyn(grid.clim.t.nat, grid.clim.t.inv, quant=0, interest=2, title= "Niche Overlap",
                        name.axis1=paste(names(nat[i])))
 }

